@@ -24,28 +24,13 @@
 #include "bacon-video-widget-properties.h"
 
 #include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <glade/glade.h>
 #include <string.h>
+#include "video-utils.h"
 
 #include "debug.h"
 
-#ifdef HAVE_GTK_ONLY
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(String) dgettext(GETTEXT_PACKAGE,String)
-#ifdef gettext_noop
-#define N_(String) gettext_noop(String)
-#else
-#define N_(String) (String)
-#endif /* gettext_noop */
-#else
-#define _(String) (String)
-#define N_(String) (String)
-#endif /* ENABLE_NLS */
-#else
-#include <libgnome/gnome-i18n.h>
-#endif /* HAVE_GTK_ONLY */
-
 struct BaconVideoWidgetPropertiesPrivate
 {
 	GladeXML *xml;
@@ -59,31 +44,7 @@ static void bacon_video_widget_properties_class_init
 static void bacon_video_widget_properties_init
 	(BaconVideoWidgetProperties *props);
 
-GtkType
-bacon_video_widget_properties_get_type (void)
-{
-	static GtkType bacon_video_widget_properties_type = 0;
-
-	if (!bacon_video_widget_properties_type) {
-		static const GTypeInfo bacon_video_widget_properties_info = {
-			sizeof (BaconVideoWidgetPropertiesClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) bacon_video_widget_properties_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL /* class_data */,
-			sizeof (BaconVideoWidgetProperties),
-			0 /* n_preallocs */,
-			(GInstanceInitFunc) bacon_video_widget_properties_init,
-		};
-
-		bacon_video_widget_properties_type = g_type_register_static
-			(GTK_TYPE_VBOX, "BaconVideoWidgetProperties",
-			 &bacon_video_widget_properties_info, (GTypeFlags)0);
-	}
-
-	return bacon_video_widget_properties_type;
-}
+G_DEFINE_TYPE(BaconVideoWidgetProperties, bacon_video_widget_properties, GTK_TYPE_VBOX)
 
 static void
 bacon_video_widget_properties_init (BaconVideoWidgetProperties *props)
@@ -107,48 +68,6 @@ bacon_video_widget_properties_finalize (GObject *object)
 	}
 }
 
-char *
-bacon_video_widget_properties_time_to_string (int time)
-{
-	char *secs, *mins, *hours, *string;
-	int sec, min, hour;
-
-	sec = time % 60;
-	time = time - sec;
-	min = (time % (60*60)) / 60;
-	time = time - (min * 60);
-	hour = time / (60*60);
-
-	hours = g_strdup_printf (ngettext ("%d hour", "%d hours", hour), hour);
-
-	mins = g_strdup_printf (ngettext ("%d minute",
-					  "%d minutes", min), min);
-
-	secs = g_strdup_printf (ngettext ("%d second",
-					  "%d seconds", sec), sec);
-
-	if (hour > 0)
-	{
-		/* hour:minutes:seconds */
-		string = g_strdup_printf (_("%s %s %s"), hours, mins, secs);
-	} else if (min > 0) {
-		/* minutes:seconds */
-		string = g_strdup_printf (_("%s %s"), mins, secs);
-	} else if (sec > 0) {
-		/* seconds */
-		string = g_strdup_printf (_("%s"), secs);
-	} else {
-		/* 0 seconds */
-		string = g_strdup (_("0 seconds"));
-	}
-
-	g_free (hours);
-	g_free (mins);
-	g_free (secs);
-
-	return string;
-}
-
 static void
 bacon_video_widget_properties_set_label (BaconVideoWidgetProperties *props,
 			       const char *name, const char *text)
@@ -224,8 +143,8 @@ bacon_video_widget_properties_set_from_current
 
 	bacon_video_widget_get_metadata (BACON_VIDEO_WIDGET (bvw),
 			BVW_INFO_DURATION, &value);
-	string = bacon_video_widget_properties_time_to_string
-		(g_value_get_int (&value));
+	string = totem_time_to_string_text
+		(g_value_get_int (&value) * 1000);
 	bacon_video_widget_properties_set_label (props, "duration", string);
 	g_free (string);
 	g_value_unset (&value);
