@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.121 2004/06/22 14:16:35 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.123 2004/07/22 17:59:01 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -115,6 +115,7 @@ static int video_props[4] = {
 	XINE_PARAM_VO_SATURATION,
 	XINE_PARAM_VO_HUE
 };
+
 static char *video_props_str[4] = {
 	GCONF_PREFIX"/brightness",
 	GCONF_PREFIX"/contrast",
@@ -130,7 +131,6 @@ struct BaconVideoWidgetPrivate {
 	xine_audio_port_t *ao_driver;
 	xine_event_queue_t *ev_queue;
 	double display_ratio;
-	gboolean started;
 
 	/* Configuration */
 	GConfClient *gc;
@@ -164,7 +164,6 @@ struct BaconVideoWidgetPrivate {
 	gboolean auto_resize;
 	int volume;
 	TvOutType tvout;
-	guint32 video_fcc, audio_fcc;
 	gboolean is_live;
 
 	GAsyncQueue *queue;
@@ -627,7 +626,7 @@ load_video_out_driver (BaconVideoWidget *bvw, gboolean null_out)
 }
 
 static xine_audio_port_t *
-load_audio_out_driver (BaconVideoWidget *bvw, GError **err)
+load_audio_out_driver (BaconVideoWidget *bvw, GError **error)
 {
 	xine_audio_port_t *ao_driver;
 	const char *audio_driver_id;
@@ -662,7 +661,7 @@ load_audio_out_driver (BaconVideoWidget *bvw, GError **err)
 
 	if (ao_driver == NULL && strcmp (audio_driver_id, "auto") != 0)
 	{
-		g_set_error (err, 0, 0,
+		g_set_error (error, BVW_ERROR, BVW_ERROR_AUDIO_PLUGIN,
 				_("Couldn't load the '%s' audio driver\n"
 					"Check that the device is not busy."),
 				audio_driver_id ? audio_driver_id : "auto" );
@@ -1263,51 +1262,66 @@ static void
 xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 {
 	char *message;
+	int num;
 	signal_data *sigdata;
 
 	message = NULL;
 
-	switch(data->type)
+	switch (data->type)
 	{
 	case XINE_MSG_NO_ERROR:
 		break;
 	case XINE_MSG_GENERAL_WARNING:
 		break;
 	case XINE_MSG_UNKNOWN_HOST:
+		num = BVW_ERROR_UNKNOWN_HOST;
 		message = g_strdup (_("The server you are trying to connect to is not known."));
 		break;
 	case XINE_MSG_UNKNOWN_DEVICE:
+		num = BVW_ERROR_INVALID_DEVICE;
 		message = g_strdup_printf (_("The device name you specified (%s) seems to be invalid."), (char *) data + data->parameters);
 		break;
 	case XINE_MSG_NETWORK_UNREACHABLE:
+		num = BVW_ERROR_NETWORK_UNREACHABLE;
 		message = g_strdup_printf (_("The server you are trying to connect to (%s) is unreachable."), (char *) data + data->parameters);
 		break;
 	case XINE_MSG_CONNECTION_REFUSED:
+		num = BVW_ERROR_CONNECTION_REFUSED;
 		message = g_strdup (_("The connection to this server was refused."));
 		break;
 	case XINE_MSG_FILE_NOT_FOUND:
+		num = BVW_ERROR_FILE_NOT_FOUND;
 		message = g_strdup (_("The specified movie could not be found."));
 		break;
 	case XINE_MSG_READ_ERROR:
+		num = BVW_ERROR_READ_ERROR;
 		message = g_strdup (_("The movie could not be read."));
 		break;
 	case XINE_MSG_LIBRARY_LOAD_ERROR:
+		num = BVW_ERROR_PLUGIN_LOAD;
 		message = g_strdup_printf (_("A problem occured while loading a library or a decoder (%s)."), (char *) data + data->parameters);
 		break;
 	case XINE_MSG_ENCRYPTED_SOURCE:
 		if (strncmp (bvw->priv->mrl, "dvd:", 4) == 0)
+		{
+			num = BVW_ERROR_DVD_ENCRYPTED;
 			message = g_strdup (_("The source seems encrypted, and can't be read. Are you trying to play an encrypted DVD without libdvdcss?"));
-		else
+		} else {
+			num = BVW_ERROR_FILE_ENCRYPTED;
 			message = g_strdup (_("This file is encrypted and cannot be played back."));
+		}
 		break;
 	case XINE_MSG_SECURITY:
+		num = BVW_ERROR_GENERIC;
 		message = g_strdup (_("For security reasons, this movie can not be played back."));
 		break;
 	case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
 		xine_stop (bvw->priv->stream);
+		num = BVW_ERROR_AUDIO_BUSY;
 		message = g_strdup (_("The audio device is busy. Is another application using it?"));
 		break;
 	case XINE_MSG_PERMISSION_ERROR:
+		num = BVW_ERROR_FILE_PERMISSION;
 		if (strncmp (bvw->priv->mrl, "file:", 5) == 0)
 			message = g_strdup (_("You are not allowed to open this file."));
 		else
@@ -1324,6 +1338,7 @@ xine_event_message (BaconVideoWidget *bvw, xine_ui_message_data_t *data)
 	sigdata = g_new0 (signal_data, 1);
 	sigdata->signal = MESSAGE_ASYNC;
 	sigdata->msg = message;
+	sigdata->num = num;
 	g_async_queue_push (bvw->priv->queue, sigdata);
 	g_idle_add ((GSourceFunc) bacon_video_widget_idle_signal, bvw);
 }
@@ -1422,7 +1437,8 @@ xine_error (BaconVideoWidget *bvw, GError **error)
 
 	if (save_data != NULL)
 	{
-		g_set_error (error, 0, 0, "%s", save_data->msg);
+		g_set_error (error, BVW_ERROR, save_data->num,
+				"%s", save_data->msg);
 		g_free (save_data->msg);
 		g_free (save_data);
 
@@ -1437,22 +1453,24 @@ xine_error (BaconVideoWidget *bvw, GError **error)
 	{
 	case XINE_ERROR_NO_INPUT_PLUGIN:
 	case XINE_ERROR_NO_DEMUX_PLUGIN:
-		g_set_error (error, 0, 0, _("There is no plugin to handle this"
-					" movie."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_NO_PLUGIN_FOR_FILE,
+				_("There is no plugin to handle this movie."));
 		break;
 	case XINE_ERROR_DEMUX_FAILED:
-		g_set_error (error, 0, 0, _("This movie is broken and can not "
-					"be played further."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_BROKEN_FILE,
+				_("This movie is broken and can not be played further."));
 		break;
 	case XINE_ERROR_MALFORMED_MRL:
-		g_set_error (error, 0, 0, _("This location is not "
-					"a valid one."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_UNVALID_LOCATION,
+				_("This location is not a valid one."));
 		break;
 	case XINE_ERROR_INPUT_FAILED:
-		g_set_error (error, 0, 0, _("This movie could not be opened."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_FILE_GENERIC,
+				_("This movie could not be opened."));
 		break;
 	default:
-		g_set_error (error, 0, 0, _("Generic Error."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_GENERIC,
+				_("Generic Error."));
 		break;
 	}
 }
@@ -1538,8 +1556,19 @@ bacon_video_widget_get_popt_table (void)
 	return (struct poptOption *) xine_options;
 }
 
+GQuark
+bacon_video_widget_error_quark (void)
+{
+	static GQuark q = 0;
+	if (q == 0)
+		q = g_quark_from_static_string ("bvw-error-quark");
+	return q;
+}
+
+
 GtkWidget *
-bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
+bacon_video_widget_new (int width, int height,
+		gboolean null_out, GError **error)
 {
 	BaconVideoWidget *bvw;
 	xine_cfg_entry_t entry;
@@ -1570,8 +1599,8 @@ bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 	bvw->priv->init_height = height;
 
 	/* load the output drivers */
-	bvw->priv->ao_driver = load_audio_out_driver (bvw, err);
-	if (err != NULL && *err != NULL)
+	bvw->priv->ao_driver = load_audio_out_driver (bvw, error);
+	if (error != NULL && *error != NULL)
 		return NULL;
 
 	bvw->priv->vo_driver = load_video_out_driver (bvw, TRUE);
@@ -1597,7 +1626,7 @@ bacon_video_widget_new (int width, int height, gboolean null_out, GError **err)
 		g_free (bvw->priv);
 		g_free (bvw);
 
-		g_set_error (err, 0, 0,
+		g_set_error (error, BVW_ERROR, BVW_ERROR_VIDEO_PLUGIN,
 				_("No video output is available. Make sure that the program is correctly installed."));
 		return NULL;
 	}
@@ -1844,26 +1873,33 @@ bacon_video_widget_get_backend_name (BaconVideoWidget *bvw)
 
 gboolean
 bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
-		GError **gerror)
+		GError **error)
 {
-	int error;
+	int err;
 
 	g_return_val_if_fail (mrl != NULL, FALSE);
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 	g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
-	bvw->priv->video_fcc = 0;
-	bvw->priv->audio_fcc = 0;
-
-	error = xine_open (bvw->priv->stream, mrl);
-	if (error == 0)
+	err = xine_open (bvw->priv->stream, mrl);
+	if (err == 0)
 	{
 		bacon_video_widget_close (bvw);
-		xine_error (bvw, gerror);
+		xine_error (bvw, error);
 		return FALSE;
 	}
 
+	if (strcmp (xine_get_meta_info (bvw->priv->stream, XINE_META_INFO_SYSTEMLAYER), "MNG") == 0 && bvw->priv->logo_mode == FALSE)
+	{
+		bacon_video_widget_close (bvw);
+		g_set_error (error, BVW_ERROR, BVW_ERROR_STILL_IMAGE,
+				_("This movie is a still image. You can open it with an image viewer."));
+		return FALSE;
+	}
+
+	g_message ("XINE_META_INFO_SYSTEMLAYER: %s", xine_get_meta_info (bvw->priv->stream, XINE_META_INFO_SYSTEMLAYER));
+
 	if (xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_HANDLED) == FALSE
 			|| (xine_get_stream_info (bvw->priv->stream,
@@ -1871,34 +1907,28 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 				&& xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_AUDIO_HANDLED) == FALSE))
 	{
-		char *fourcc_str, *name;
+		char *name;
 
 		g_signal_emit (G_OBJECT (bvw),
 				bvw_table_signals[GOT_METADATA], 0, NULL);
 
-		bvw->priv->video_fcc = xine_get_stream_info (bvw->priv->stream,
-				XINE_STREAM_INFO_VIDEO_FOURCC);
-		fourcc_str = get_fourcc_string (bvw->priv->video_fcc);
 		name = g_strdup (xine_get_meta_info (bvw->priv->stream,
 				XINE_META_INFO_VIDEOCODEC));
-
-		/* Only change the audio_fcc if we're on x86 */
-		if (xine_get_stream_info
-				(bvw->priv->stream,
-				 XINE_STREAM_INFO_AUDIO_HANDLED) == FALSE)
+		if (name == NULL)
 		{
-			bvw->priv->audio_fcc = xine_get_stream_info
-				(bvw->priv->stream,
-				 XINE_STREAM_INFO_AUDIO_FOURCC);
+			guint32 video_fcc;
+
+			video_fcc = xine_get_stream_info (bvw->priv->stream,
+					XINE_STREAM_INFO_VIDEO_FOURCC);
+			name = get_fourcc_string (video_fcc);
 		}
 
 		bacon_video_widget_close (bvw);
 
-		g_set_error (gerror, 0, 0,
+		g_set_error (error, BVW_ERROR, BVW_ERROR_CODEC_NOT_HANDLED,
 				_("Video codec '%s' is not handled. You might need to install additional plugins to be able to play some types of movies"),
-				name ? name : fourcc_str);
+				name);
 
-		g_free (fourcc_str);
 		g_free (name);
 
 		return FALSE;
@@ -1914,7 +1944,7 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 
 		bacon_video_widget_close (bvw);
 
-		g_set_error (gerror, 0, 0,
+		g_set_error (error, BVW_ERROR, BVW_ERROR_AUDIO_ONLY,
 				_("This is an audio-only file, and there is no audio output available."));
 
 		return FALSE;
@@ -1939,7 +1969,6 @@ bacon_video_widget_play (BaconVideoWidget *bvw, GError **gerror)
 	g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
 	g_return_val_if_fail (bvw->priv->xine != NULL, -1);
 
-	bvw->priv->started = TRUE;
 	error = 1;
 
 	if (bvw->priv->seeking == 1)
@@ -2903,7 +2932,7 @@ bacon_video_widget_set_aspect_ratio (BaconVideoWidget *bvw,
 	xine_set_param (bvw->priv->stream, XINE_PARAM_VO_ASPECT_RATIO, ratio);
 }
 
-int
+BaconVideoWidgetAspectRatio
 bacon_video_widget_get_aspect_ratio (BaconVideoWidget *bvw)
 {
 	g_return_val_if_fail (bvw != NULL, 0);
@@ -3184,9 +3213,6 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw,
 		integer = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_HEIGHT);
 		break;
-	case BVW_INFO_VIDEO_FOURCC:
-		integer = (guint32) bvw->priv->video_fcc;
-		break;
 	case BVW_INFO_FPS:
 		if (xine_get_stream_info (bvw->priv->stream,
 					XINE_STREAM_INFO_FRAME_DURATION) != 0)
@@ -3202,9 +3228,6 @@ bacon_video_widget_get_metadata_int (BaconVideoWidget *bvw,
 		integer = xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_AUDIO_BITRATE) / 1000;
 		break;
-	 case BVW_INFO_AUDIO_FOURCC:
-		integer = (guint32) bvw->priv->audio_fcc;
-		break;
 	 default:
 		g_assert_not_reached ();
 	 }
@@ -3268,10 +3291,8 @@ bacon_video_widget_get_metadata (BaconVideoWidget *bvw,
 	case BVW_INFO_DURATION:
 	case BVW_INFO_DIMENSION_X:
 	case BVW_INFO_DIMENSION_Y:
-	case BVW_INFO_VIDEO_FOURCC:
 	case BVW_INFO_FPS:
 	case BVW_INFO_BITRATE:
-	case BVW_INFO_AUDIO_FOURCC:
 		bacon_video_widget_get_metadata_int (bvw, type, value);
 		break;
 	case BVW_INFO_HAS_VIDEO:
@@ -3390,7 +3411,8 @@ bacon_video_widget_can_get_frames (BaconVideoWidget *bvw, GError **error)
 	if (xine_get_status (bvw->priv->stream) != XINE_STATUS_PLAY
 			&& bvw->priv->logo_mode == FALSE)
 	{
-		g_set_error (error, 0, 0, _("Movie is not playing."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_CANNOT_CAPTURE,
+				_("Movie is not playing."));
 		return FALSE;
 	}
 
@@ -3398,14 +3420,16 @@ bacon_video_widget_can_get_frames (BaconVideoWidget *bvw, GError **error)
 				XINE_STREAM_INFO_HAS_VIDEO) == FALSE
 			&& bvw->priv->using_vfx == FALSE)
 	{
-		g_set_error (error, 0, 0, _("No video to capture."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_CANNOT_CAPTURE,
+				_("No video to capture."));
 		return FALSE;
 	}
 
 	if (xine_get_stream_info (bvw->priv->stream,
 				XINE_STREAM_INFO_VIDEO_HANDLED) == FALSE)
 	{
-		g_set_error (error, 0, 0, _("Video codec is not handled."));
+		g_set_error (error, BVW_ERROR, BVW_ERROR_CANNOT_CAPTURE,
+				_("Video codec is not handled."));
 		return FALSE;
 	}
 
