@@ -34,10 +34,9 @@ enum
   ARG_VISIBLE_CURSOR,
   ARG_LOGO_FOCUSED,
   ARG_EVENT_CATCHER,
-  ARG_XID,
   ARG_SOURCE_WIDTH,
   ARG_SOURCE_HEIGHT,
-  ARG_LOGO,
+  ARG_LOGO
 };
 
 struct _GstVideoWidgetPrivate
@@ -47,8 +46,6 @@ struct _GstVideoWidgetPrivate
   GdkWindow *video_window;
 
   GdkPixbuf *logo_pixbuf;
-
-  gulong xembed_xid;
   
   guint video_window_width;
   guint video_window_height;
@@ -56,8 +53,8 @@ struct _GstVideoWidgetPrivate
   guint source_width;
   guint source_height;
 
-  guint width_mini;
-  guint height_mini;
+  gint width_mini;
+  gint height_mini;
 
   gboolean auto_resize;
   gboolean cursor_visible;
@@ -135,31 +132,27 @@ gst_video_widget_reorder_windows (GstVideoWidget * vw)
 {
   g_return_if_fail (vw != NULL);
   g_return_if_fail (GST_IS_VIDEO_WIDGET (vw));
-
-  if (vw->priv->event_catcher)
-    {
-      if (GDK_IS_WINDOW (vw->priv->event_window))
-	gdk_window_raise (vw->priv->event_window);
-    }
-  else
-    {
-      if (GDK_IS_WINDOW (vw->priv->video_window))
-	gdk_window_raise (vw->priv->video_window);
-    }
-
+  
   if ((vw->priv->logo_focused) && (GDK_IS_WINDOW (vw->priv->video_window)))
     {
       gdk_window_hide (vw->priv->video_window);
     }
   else if ((!vw->priv->logo_focused) &&
-	   (GDK_IS_WINDOW (vw->priv->video_window)))
+           (GDK_IS_WINDOW (vw->priv->video_window)))
     {
       gdk_window_show (vw->priv->video_window);
     }
-  else
+
+  if (GDK_IS_WINDOW (vw->priv->video_window))
+    gdk_window_raise (vw->priv->video_window);
+  
+  if (vw->priv->event_catcher)
     {
-      gtk_widget_queue_draw (GTK_WIDGET (vw));
+      if (GDK_IS_WINDOW (vw->priv->event_window))
+        gdk_window_raise (vw->priv->event_window);
     }
+  
+  gtk_widget_queue_draw (GTK_WIDGET (vw));
 }
 
 /* =========================================== */
@@ -180,7 +173,7 @@ gst_video_widget_realize (GtkWidget * widget)
   g_return_if_fail (vw != NULL);
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-
+  
   /* Creating our widget's window */
 
   attributes.window_type = GDK_WINDOW_CHILD;
@@ -201,6 +194,25 @@ gst_video_widget_realize (GtkWidget * widget)
 
   gdk_window_set_user_data (widget->window, widget);
 
+  /* Creating our video window */
+
+  attributes.window_type = GDK_WINDOW_CHILD;
+  attributes.x = 0;
+  attributes.y = 0;
+  attributes.width = widget->allocation.width;
+  attributes.height = widget->allocation.height;
+  attributes.wclass = GDK_INPUT_OUTPUT;
+  attributes.event_mask = GDK_EXPOSURE_MASK;
+
+  attributes_mask = GDK_WA_X | GDK_WA_Y;
+
+  vw->priv->video_window = gdk_window_new (widget->window,
+				           &attributes, attributes_mask);
+
+  gdk_window_set_user_data (vw->priv->video_window, widget);
+
+  gdk_window_show (vw->priv->video_window);
+  
   /* Creating our event window */
 
   attributes.window_type = GDK_WINDOW_CHILD;
@@ -281,83 +293,82 @@ gst_video_widget_expose (GtkWidget * widget, GdkEventExpose * event)
   g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GST_IS_VIDEO_WIDGET (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
-
+  
   vw = GST_VIDEO_WIDGET (widget);
 
   if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget))
     {
-
       if ((vw->priv->logo_focused) && (vw->priv->logo_pixbuf))
-	{
-	  GdkPixbuf *frame;
-	  guchar *pixels;
-	  int rowstride;
-	  gint width, height, alloc_width, alloc_height, logo_x, logo_y;
-	  gfloat width_ratio, height_ratio;
+        {
+          GdkPixbuf *frame;
+          guchar *pixels;
+          int rowstride;
+          gint width, height, alloc_width, alloc_height, logo_x, logo_y;
+          gfloat width_ratio, height_ratio;
 
-	  frame = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-				  FALSE, 8, widget->allocation.width,
-				  widget->allocation.height);
+          frame = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+                                  FALSE, 8, widget->allocation.width,
+                                  widget->allocation.height);
 
-	  width = gdk_pixbuf_get_width (vw->priv->logo_pixbuf);
-	  height = gdk_pixbuf_get_height (vw->priv->logo_pixbuf);
-	  alloc_width = widget->allocation.width;
-	  alloc_height = widget->allocation.height;
+          width = gdk_pixbuf_get_width (vw->priv->logo_pixbuf);
+          height = gdk_pixbuf_get_height (vw->priv->logo_pixbuf);
+          alloc_width = widget->allocation.width;
+          alloc_height = widget->allocation.height;
 
-	  /* Checking if allocated space is smaller than our logo */
+          /* Checking if allocated space is smaller than our logo */
 
-	  if ((alloc_width < width) || (alloc_height < height))
-	    {
-	      width_ratio = (gfloat) alloc_width / (gfloat) width;
-	      height_ratio = (gfloat) alloc_height / (gfloat) height;
-	      width_ratio = MIN (width_ratio, height_ratio);
-	      height_ratio = width_ratio;
-	    }
-	  else
-	    width_ratio = height_ratio = 1.0;
+          if ((alloc_width < width) || (alloc_height < height))
+            {
+              width_ratio = (gfloat) alloc_width / (gfloat) width;
+              height_ratio = (gfloat) alloc_height / (gfloat) height;
+              width_ratio = MIN (width_ratio, height_ratio);
+              height_ratio = width_ratio;
+            }
+          else
+            width_ratio = height_ratio = 1.0;
 
-	  logo_x = (alloc_width / 2) - (width * width_ratio / 2);
-	  logo_y = (alloc_height / 2) - (height * height_ratio / 2);
+          logo_x = (alloc_width / 2) - (width * width_ratio / 2);
+          logo_y = (alloc_height / 2) - (height * height_ratio / 2);
 
-	  /* Scaling to available space */
+          /* Scaling to available space */
 
-	  gdk_pixbuf_composite (vw->priv->logo_pixbuf,
-				frame,
-				0, 0,
-				alloc_width, alloc_height,
-				logo_x, logo_y,
-				width_ratio, height_ratio,
-				GDK_INTERP_BILINEAR, 255);
+          gdk_pixbuf_composite (vw->priv->logo_pixbuf,
+                                frame,
+                                0, 0,
+                                alloc_width, alloc_height,
+                                logo_x, logo_y,
+                                width_ratio, height_ratio,
+                                GDK_INTERP_BILINEAR, 255);
 
-	  /* Drawing our frame */
+          /* Drawing our frame */
 
-	  rowstride = gdk_pixbuf_get_rowstride (frame);
+          rowstride = gdk_pixbuf_get_rowstride (frame);
 
-	  pixels = gdk_pixbuf_get_pixels (frame) +
-	    rowstride * event->area.y + event->area.x * 3;
+          pixels = gdk_pixbuf_get_pixels (frame) +
+            rowstride * event->area.y + event->area.x * 3;
+          
+          gdk_draw_rgb_image_dithalign (widget->window,
+                                        widget->style->black_gc,
+                                        event->area.x, event->area.y,
+                                        event->area.width, event->area.height,
+                                        GDK_RGB_DITHER_NORMAL, pixels,
+                                        rowstride, event->area.x,
+                                        event->area.y);
 
-	  gdk_draw_rgb_image_dithalign (widget->window,
-					widget->style->black_gc,
-					event->area.x, event->area.y,
-					event->area.width, event->area.height,
-					GDK_RGB_DITHER_NORMAL, pixels,
-					rowstride, event->area.x,
-					event->area.y);
-
-	  g_object_unref (frame);
-	}
+          g_object_unref (frame);
+        }
       else
-	{
-	  gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE,
-			      event->area.x, event->area.y,
-			      event->area.width, event->area.height);
-	}
+        {
+          gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE,
+                              event->area.x, event->area.y,
+                              event->area.width, event->area.height);
+        }
 
     }
     
   if (GDK_IS_WINDOW (vw->priv->video_window))
     {
-      gint pos_x, pos_y, width, height, depth;
+      guint pos_x, pos_y, width, height, depth;
       gdk_window_get_geometry (vw->priv->video_window, &pos_x, &pos_y,
                                &width, &height, &depth);
       if ((width != vw->priv->video_window_width) ||
@@ -446,7 +457,7 @@ gst_video_widget_allocate (GtkWidget *widget, GtkAllocation *allocation)
 {
   GstVideoWidget *vw;
   gfloat temp, scale_factor = 1.0;
-  guint width, height;
+  gint width, height;
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GST_IS_VIDEO_WIDGET (widget));
@@ -529,7 +540,17 @@ gst_video_widget_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	gdk_window_move_resize (vw->priv->event_window,
 				0, 0, allocation->width, allocation->height);
 
-      vw->priv->video_window_width = width;
+      /* X windows can only be resized to 1,1 not 0,0 so we have to handle
+         the case where source size is 0,0 (not set basically). So instead
+         of setting the video window to 0,0 we set it to 1,1 to match with
+         what X will really do. If we don't do that we have an infinite loop
+         with the expose event reacting on video window geometry changes. */
+      if (!width)
+        width = 1;
+      if (!height)
+        height = 1;
+      
+      vw->priv->video_window_width = width; 
       vw->priv->video_window_height = height;
       
       if (GDK_IS_WINDOW (vw->priv->video_window))
@@ -580,9 +601,6 @@ gst_video_widget_set_property (GObject * object, guint prop_id,
       vw->priv->event_catcher = g_value_get_boolean (value);
       gst_video_widget_reorder_windows (vw);
       break;
-    case ARG_XID:
-      gst_video_widget_set_xembed_xid (vw, g_value_get_ulong (value));
-      break;
     case ARG_SOURCE_WIDTH:
       vw->priv->source_width = g_value_get_int (value);
       break;
@@ -633,9 +651,6 @@ gst_video_widget_get_property (GObject * object, guint prop_id,
     case ARG_EVENT_CATCHER:
       g_value_set_boolean (value, vw->priv->event_catcher);
       break;
-    case ARG_XID:
-      g_value_set_ulong (value, vw->priv->xembed_xid);
-      break;
     case ARG_SOURCE_WIDTH:
       g_value_set_int (value, vw->priv->source_width);
       break;
@@ -714,14 +729,6 @@ gst_video_widget_class_init (GstVideoWidgetClass * klass)
 							 TRUE,
 							 G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class,
-				   ARG_XID,
-				   g_param_spec_ulong ("video_xid",
-						       "video window xid",
-						       "Video playback Xwindow XID",
-						       0, G_MAXLONG, 1,
-						       G_PARAM_READWRITE));
-
   g_object_class_install_property (gobject_class,
 				   ARG_SOURCE_WIDTH,
 				   g_param_spec_int ("source_width",
@@ -743,8 +750,7 @@ gst_video_widget_class_init (GstVideoWidgetClass * klass)
 				   g_param_spec_object ("logo",
 							"Logo",
 							"Picture that should appear as a logo when no video",
-							gdk_pixbuf_get_type
-							(),
+							gdk_pixbuf_get_type (),
 							G_PARAM_READWRITE));
 
   widget_class->realize = gst_video_widget_realize;
@@ -764,7 +770,8 @@ gst_video_widget_init (GstVideoWidget * vw)
   vw->priv = g_new0 (GstVideoWidgetPrivate, 1);
 
   GTK_WIDGET_SET_FLAGS (GTK_WIDGET (vw), GTK_CAN_FOCUS);
-
+  GTK_WIDGET_UNSET_FLAGS (GTK_WIDGET (vw), GTK_DOUBLE_BUFFERED);
+  
   vw->priv->source_width = 0;
   vw->priv->source_height = 0;
   vw->priv->width_mini = 0;
@@ -774,7 +781,7 @@ gst_video_widget_init (GstVideoWidget * vw)
   vw->priv->scale_override = FALSE;
   vw->priv->cursor_visible = TRUE;
   vw->priv->event_catcher = TRUE;
-  vw->priv->logo_focused = TRUE;
+  vw->priv->logo_focused = FALSE;
   vw->priv->event_window = NULL;
   vw->priv->video_window = NULL;
   vw->priv->logo_pixbuf = NULL;
@@ -786,101 +793,19 @@ gst_video_widget_init (GstVideoWidget * vw)
 /*                                                               */
 /* ============================================================= */
 
-/**
- * gst_video_widget_set_xembed_xid:
- * @vw: a #GstVideoWidget
- * @xid: the window ID of an existing window.
- * 
- * Reparents a pre-existing video window into a #GstVideoWidget. This is
- *  meant to embed a foreign Xwindow created by xvideosink for example.
- *
- * Remember you can set this value trough the "video_xid" property. This will
- *  trigger the embedding the same way than calling this method.
- *
- **/
-void
-gst_video_widget_set_xembed_xid (GstVideoWidget * vw, gulong xid)
+GdkWindow *
+gst_video_widget_get_video_window (GstVideoWidget * vw)
 {
-  GtkWidget *widget = GTK_WIDGET (vw);
-
-  gdk_threads_enter ();
-
-  vw->priv->logo_focused = FALSE;
-
-  if (GDK_IS_WINDOW (vw->priv->video_window))
-    {
-      gdk_window_destroy (vw->priv->video_window);
-      vw->priv->video_window = NULL;
-    }
-
-  vw->priv->video_window = gdk_window_foreign_new (xid);
-
-  if (vw->priv->video_window)
-    {
-      gint video_x, video_y, video_width, video_height, video_depth;
-
-      gdk_window_reparent (vw->priv->video_window, widget->window, 0, 0);
-
-      gdk_window_show (vw->priv->video_window);
-
-      if (vw->priv->event_catcher)
-	gdk_window_raise (vw->priv->event_window);
-
-      vw->priv->xembed_xid = xid;
-
-      gtk_widget_queue_resize (GTK_WIDGET (vw));
-
-    }
-  else
-    g_warning ("Trying to embed a window which has been destroyed");
-
-  gdk_threads_leave ();
-}
-
-/**
- * gst_video_widget_get_xembed_xid:
- * @vw: a #GstVideoWidget
- * 
- * Get current embeded window xid.
- *
- * Remember you can get this value trough the "video_xid" property.
- *
- * Return value: a #gulong referencing embeded video window.
- **/
-gulong
-gst_video_widget_get_xembed_xid (GstVideoWidget * vw)
-{
-  g_return_val_if_fail (vw != NULL, 0);
-  g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), 0);
-  return vw->priv->xembed_xid;
-}
-
-gboolean
-gst_video_widget_destroy_embedded_window (GstVideoWidget * vw)
-{
-  g_return_val_if_fail (vw != NULL, FALSE);
-  g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-  
-  if (vw->priv->xembed_xid)
-    {
-      gdk_threads_enter ();
-      gdk_window_destroy (vw->priv->video_window);
-      gdk_threads_leave ();
-      vw->priv->video_window = NULL;
-    }
-  
-  vw->priv->logo_focused = TRUE;
-
-  gst_video_widget_reorder_windows (vw);
-    
-  return TRUE;
+  g_return_val_if_fail (vw != NULL, NULL);
+  g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), NULL);
+  return vw->priv->video_window;
 }
 
 /**
  * gst_video_widget_set_source_size:
  * @vw: a #GstVideoWidget
- * @width: a #gint indicating source video's width.
- * @height: a #gint indicating source video's height.
+ * @width: a #guint indicating source video's width.
+ * @height: a #guint indicating source video's height.
  * 
  * Set video source size of a #GstVideoWidget and queue a resize request
  *  for the widget.
@@ -894,15 +819,18 @@ gst_video_widget_destroy_embedded_window (GstVideoWidget * vw)
  * Return value: a #gboolean indicating wether the call succeeded or not.
  **/
 gboolean
-gst_video_widget_set_source_size (GstVideoWidget * vw, gint width,
-				  gint height)
+gst_video_widget_set_source_size (GstVideoWidget * vw, guint width,
+				  guint height)
 {
   g_return_val_if_fail (vw != NULL, FALSE);
   g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
   
-  vw->priv->source_width = width;
-  vw->priv->source_height = height;
-  gtk_widget_queue_resize (GTK_WIDGET (vw));
+  if (vw->priv->source_width != width || vw->priv->source_height != height)
+    {
+      vw->priv->source_width = width;
+      vw->priv->source_height = height;
+      gtk_widget_queue_resize (GTK_WIDGET (vw));
+    }
 
   return TRUE;
 }
@@ -910,8 +838,8 @@ gst_video_widget_set_source_size (GstVideoWidget * vw, gint width,
 /**
  * gst_video_widget_get_source_size:
  * @socket_: a #GstVideoWidget
- * @width: a pointer to a #gint where source video's width will be put.
- * @height: a pointer to a #gint where source video's height will be put.
+ * @width: a pointer to a #guint where source video's width will be put.
+ * @height: a pointer to a #guint where source video's height will be put.
  * 
  * Fills @width and @height with source video's dimensions.
  *
@@ -921,8 +849,8 @@ gst_video_widget_set_source_size (GstVideoWidget * vw, gint width,
  * Return value: a #gboolean indicating wether the call succeeded or not.
  **/
 gboolean
-gst_video_widget_get_source_size (GstVideoWidget * vw, gint * width,
-				  gint * height)
+gst_video_widget_get_source_size (GstVideoWidget * vw, guint * width,
+				  guint * height)
 {
   g_return_val_if_fail (vw != NULL, FALSE);
   g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
@@ -1049,7 +977,7 @@ gst_video_widget_set_logo_focus (GstVideoWidget * vw, gboolean focused)
 {
   g_return_val_if_fail (vw != NULL, FALSE);
   g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), FALSE);
-
+  
   vw->priv->logo_focused = focused;
 
   gst_video_widget_reorder_windows (vw);
@@ -1275,8 +1203,8 @@ gst_video_widget_set_logo (GstVideoWidget * vw, GdkPixbuf * logo_pixbuf)
 GdkPixbuf *
 gst_video_widget_get_logo (GstVideoWidget * vw)
 {
-  g_return_if_fail (vw != NULL);
-  g_return_if_fail (GST_IS_VIDEO_WIDGET (vw));
+  g_return_val_if_fail (vw != NULL, NULL);
+  g_return_val_if_fail (GST_IS_VIDEO_WIDGET (vw), NULL);
   return vw->priv->logo_pixbuf;
 }
 
