@@ -571,7 +571,7 @@ totem_playlist_add_files (GtkWidget *widget, TotemPlaylist *playlist)
 	GtkWidget *fs;
 	int response;
 
-	fs = gtk_file_chooser_dialog_new (_("Select files"),
+	fs = gtk_file_chooser_dialog_new (_("Select Movies or Playlists"),
 			GTK_WINDOW (playlist), GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_ADD, GTK_RESPONSE_ACCEPT,
@@ -774,6 +774,8 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		char *filename, *tmp;
+		GnomeVFSResult res;
+		GnomeVFSFileInfo info;
 
 		filename = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
 
@@ -788,7 +790,8 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 				G_DIR_SEPARATOR_S, NULL);
 		g_free (tmp);
 
-		if (g_file_test (filename, G_FILE_TEST_EXISTS) != FALSE)
+		res = gnome_vfs_get_file_info (filename, &info, GNOME_VFS_FILE_INFO_DEFAULT);
+		if (res != GNOME_VFS_ERROR_NOT_FOUND && res == GNOME_VFS_OK)
 		{
 			GtkWidget *dialog;
 
@@ -1524,10 +1527,19 @@ gboolean
 totem_playlist_add_mrl (TotemPlaylist *playlist, const char *mrl,
 		const char *display_name)
 {
+	TotemPlParserResult res;
+
 	g_return_val_if_fail (mrl != NULL, FALSE);
 
-	if (totem_pl_parser_parse (playlist->_priv->parser, mrl, TRUE) != TOTEM_PL_PARSER_RESULT_SUCCESS)
+	res = totem_pl_parser_parse (playlist->_priv->parser, mrl, TRUE);
+	if (res == TOTEM_PL_PARSER_RESULT_UNHANDLED)
 		return totem_playlist_add_one_mrl (playlist, mrl, display_name);
+	if (res == TOTEM_PL_PARSER_RESULT_ERROR)
+	{
+		totem_playlist_error (_("Playlist error"), _("The playlist '%s' could not be parsed, it might be damaged."), playlist);
+		return FALSE;
+	}
+
 	return TRUE;
 }
 
